/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c)  2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QList>
#include <QDate>
#include "types.h"

QT_BEGIN_NAMESPACE
//class QMdiArea;
class QAction;
class QMenu;
class QToolBar;
class QLabel;
class QPrinter;

QT_END_NAMESPACE

namespace View
{

    namespace Management
    {
       class RegistModel;
       class RegistDialog;
       class RegCEl;
    }


    class MainWindow : public QMainWindow
    {
        Q_OBJECT
    public:
        MainWindow();
        ~MainWindow();

    protected:
        void closeEvent(QCloseEvent *event);

    private slots:
        void connectStorage();
        void disconnectStorage();
        void importStorage();
        void exportStorage();
        void manageRegist();
        void about();


    private:
        void createWidgets();
        void createCentralWidget();
        void createActions();
        void createMenus();
        void createToolBar();
        void createStatusBar();

        void createConnections();
        void setStorageConnected(bool connected = true);

        bool verifyImportStorage();
        bool verifyExit();

        QAction *_connectStorageAct;
        QAction *_disconnectStorageAct;

        QAction *_importStorageAct;
        QAction *_exportStorageAct;

        QAction *_exitAct;
        QAction *_aboutAct;
        QAction *_aboutQtAct;

        QMenu *_actionMenu;
        QMenu *_helpMenu;

        QToolBar *_actionToolBar;
        QToolBar *_action1ToolBar;
        QToolBar *_helpToolBar;

        QLabel *_storageIconLabel;
        QLabel *_storageStateLabel;

        bool _connected;

        View::Management::RegistModel *_registModel;
        View::Management::RegistDialog *_registDialog;
        View::Management::RegCEl *_regcelWidget;
    };
}

#endif // MAINWINDOW_H
