/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include <QApplication>
#include <QTranslator>
#include <QLocale>
#include <iostream>
#include "mainwindow.h"
#include "persistencemanager.h"
#include "sqlagent.h"
#include "global.h"

void setUpApplication(QApplication *app, QTranslator *translator);
bool verifyConfig();
bool verifyStorage();
bool initApplication(View::MainWindow *mainWindow);


int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    QTranslator translator;


    setUpApplication(&app, &translator);

    std::cout << QString("%1 %2").arg(app.applicationName()).arg(app.applicationVersion()).toStdString() << std::endl;
    std::cout << QObject::trUtf8("Iniciando ...").toStdString() << std::endl << std::endl;

    if(!verifyConfig()) {
        std::cout << QObject::trUtf8("Configuracion          :  Error  :  La aplicacion sera cerrada").toStdString() << std::endl;
        return 1;
    } else
        std::cout << QObject::trUtf8("Configuracion          :  OK").toStdString() << std::endl;

    if(!verifyStorage()) {
        std::cout << QObject::trUtf8("Almacenamiento         :  Error  :  La aplicacion sera cerrada").toStdString() << std::endl;
        return 1;
    } else
        std::cout << QObject::trUtf8("Almacenamiento         :  OK").toStdString() << std::endl;

    View::MainWindow mainWindow;
    mainWindow.show();

    std::cout << std::endl << QObject::trUtf8("Ejecutando ...").toStdString() << std::endl;

    return app.exec();
}

void setUpApplication(QApplication *app, QTranslator *translator)
{
    app -> setOrganizationName(ORGANIZATION_NAME);
    app -> setOrganizationDomain(ORGANIZATION_DOMAIN);
    app -> setApplicationName(APPLICATION_NAME);
    app -> setApplicationVersion(APPLICATION_VERSION);



    translator -> load("regcel_" + QLocale::system().name(), ":/translations");
    app -> installTranslator(translator);
}

bool verifyConfig()
{
    if(!Persistence::Manager::existsConfig() && !Persistence::Manager::createConfig())
            return false;

    return true;
}

bool verifyStorage()
{
    if(!Persistence::Manager::existsStorage())
        return Persistence::Manager::createStorage();

    return true;
}

bool initApplication(View::MainWindow *mainWindow)
{
    return  mainWindow;
}
