/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include "registmanager.h"
#include "sqlagent.h"
#include "types.h"

bool Model::Management::RegistManager::create(const Model::Domain::Regist &regist)
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("INSERT INTO regist VALUES(%1, '%2', '%3', %4, %5, %6, %7)")
                      .arg(regist.id())
                      .arg(regist.date().toString(DATE_FORMAT))
                      .arg(regist.time().toString(TIME_FORMAT))
                      .arg(regist.lecti())
                      .arg(regist.lectf())
                      .arg(regist.kwh())
                      .arg(regist.import());

    return agent -> insert(sql);
}

bool Model::Management::RegistManager::modify(const Model::Domain::Regist &regist)
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("UPDATE regist SET date='%2', time='%3', leci=%4, "
                          "lecf=%5, kwh=%6, import=%7 WHERE id=%1")
                      .arg(regist.id())
                      .arg(regist.date().toString(DATE_FORMAT))
                      .arg(regist.time().toString(TIME_FORMAT))
                      .arg(regist.lecti())
                      .arg(regist.lectf())
                      .arg(regist.kwh())
                      .arg(regist.import());


    return agent -> update(sql);
}

bool Model::Management::RegistManager::remove(int id)
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("DELETE FROM regist WHERE id=%1").arg(id);

    return agent -> _delete(sql);
}

Model::Domain::Regist *Model::Management::RegistManager::get(int id)
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("SELECT * FROM regist WHERE id=%1").arg(id);
    QVector<QVector<QVariant> > *result = agent -> select(sql);
    Model::Domain::Regist *regist = 0;

    if(!(result -> isEmpty())) {
        QDate date                       = QDate::fromString((result -> at(0)).at(1).toString(), DATE_FORMAT);
        QTime time                       = QTime::fromString((result -> at(0)).at(2).toString(), TIME_FORMAT);
        int lecti                           = (result -> at(0)).at(3).toInt();
        int lectf                           = (result -> at(0)).at(4).toInt();
        int kwh                           = (result -> at(0)).at(5).toInt();
        double import                       = (result -> at(0)).at(6).toDouble();

        regist = new Model::Domain::Regist(id);
        regist->setDate(date);
        regist->setTime(time);
        regist->setLectI(lecti);
        regist->setLectF(lectf);
        regist->setKwh(kwh);
        regist->setImport(import);
    }

    delete result;

    return regist;
}

QList<Model::Domain::Regist *> *Model::Management::RegistManager::getAll()
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("SELECT * FROM regist");
    QVector<QVector<QVariant> > *result = agent -> select(sql);
    QList<Model::Domain::Regist *> *regists = new QList<Model::Domain::Regist *>;

    foreach(QVector<QVariant> row, *result) {
        int id                           = row.at(0).toInt();
        QDate date                       = QDate::fromString(row.at(1).toString(), DATE_FORMAT);
        QTime time                       = QTime::fromString(row.at(2).toString(), TIME_FORMAT);
        int lecti                        = row.at(3).toInt();
        int lectf                        = row.at(4).toInt();
        int kwh                          = row.at(5).toInt();
        double import                    = row.at(6).toDouble();
        
        Model::Domain::Regist *regist = new Model::Domain::Regist(id);
        regist->setDate(date);
        regist->setTime(time);
        regist->setLectI(lecti);
        regist->setLectF(lectf);
        regist->setKwh(kwh);
        regist->setImport(import);
        regists -> push_back(regist);

    }

    delete result;

    return regists;
}

int Model::Management::RegistManager::getId()
{
    Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
    QString sql = QString("SELECT count(*) FROM regist");
    QVector<QVector<QVariant> > *result = agent -> select(sql);

    if(!(result -> isEmpty())) {
        int count = (result -> at(0)).at(0).toInt();
        delete result;
        if(count == 0)
            return 1;
        else {
            sql = QString("SELECT max(id) FROM regist");
            result = agent -> select(sql);
            int id = (result -> at(0)).at(0).toInt();
            delete result;
            return id + 1;
        }
    }

    delete result;

    return NO_ID;
}

