/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include <QtGui>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>
#include <QtSql>
#include "regcelwidget.h"
#include "mainwindow.h"
#include "registdialog.h"
#include "persistencemanager.h"
#include "registmodel.h"
#include "registmanager.h"
#include "regist.h"

#include <QtGui>


View::Management::RegCEl::RegCEl(QWidget *parent)
    : QWidget(parent)

{
    //createRegistWidgets();
    
    createDateGroupBox();
    createDetailsGroupBox();
    createRegistGroupBox();
    createConnections();

    QGridLayout *layout = new QGridLayout;
    layout->addWidget(date_GroupBox, 0, 0);
    layout->addWidget(detailsGroupBox, 0, 1);
    layout->addWidget(registGroupBox, 1, 0, 1, 0);
    setLayout(layout);

    date_Layout->setRowMinimumHeight(0, calendar->sizeHint().height());
    date_Layout->setColumnMinimumWidth(0, calendar->sizeHint().width());



}
View::Management::RegCEl::~RegCEl()
{
    delete _registModel;
}

void View::Management::RegCEl::rowSelectionChangedOnRegistsTableView()
{
    int row = _registsTableView -> currentIndex().row();
    _modRegistButton -> setEnabled(row != -1);
    _delRegistButton -> setEnabled(row != -1);
    
    if (row != -1){
       Model::Domain::Regist *registdetail = _registModel -> regists() -> at(row);

       lect_inicial->display(registdetail->lecti());
       lect_final->display(registdetail->lectf());
       kw_h->display(registdetail->kwh());
       importe_lb->setText(QString("%1 $").arg(QString::number(registdetail -> import(), 'f', MAX_MONEY_PRECISION)));
       aprox_men_lb->setText(QString("%1 $").arg(QString::number(registdetail -> import() * 30 , 'f', MAX_MONEY_PRECISION)));
   }
 }

void View::Management::RegCEl::addRegist()
{
    Model::Domain::Regist *regist = new Model::Domain::Regist;
    RegistDialog dialog(regist, this);

    if(dialog.exec()) {
        if(Model::Management::RegistManager::create(*regist)) {
            int row = _registModel -> rowCount(QModelIndex());
            _registModel -> insertRegist(row, regist);

        } else {
            QMessageBox::critical(this, trUtf8("Fatal Error"),
                                  trUtf8("Error al añadir el registro"),
                                  QMessageBox::Ok);
            delete regist;
        }

        rowSelectionChangedOnRegistsTableView();
    } else
        delete regist;
}

void View::Management::RegCEl::modRegist()
{
    int row = _registsTableView -> currentIndex().row();
    Model::Domain::Regist *regist = _registModel -> regists() -> at(row);
    RegistDialog dialog(regist, this);

    if(dialog.exec()) {
        if(Model::Management::RegistManager::modify(*regist))
            _registModel -> modifyRegist(row);
        else
            QMessageBox::critical(this, trUtf8("Fatal Error"),
                                  trUtf8("Error al modificar el registro"),
                                  QMessageBox::Ok);

        rowSelectionChangedOnRegistsTableView();
    }
}

void View::Management::RegCEl::delRegist()
{
    if(!verifyDeleteRegist())
        return;

    int row = _registsTableView -> currentIndex().row();
    Model::Management::RegistManager::remove(_registModel -> regists() -> at(row) -> id());
    _registModel -> removeRegist(row);
    rowSelectionChangedOnRegistsTableView();
}


void View::Management::RegCEl::createDateGroupBox()
{
    date_GroupBox = new QGroupBox(trUtf8("Fecha"));

    calendar = new QCalendarWidget;
    calendar->setMinimumDate(QDate(2012, 1, 1));
    calendar->setMaximumDate(QDate(3000, 1, 1));
    calendar->setGridVisible(true);

    date_Layout = new QGridLayout;
    date_Layout->addWidget(calendar, 0, 0, Qt::AlignCenter);
    date_GroupBox->setLayout(date_Layout);
}

void View::Management::RegCEl::createDetailsGroupBox()
{
    detailsGroupBox = new QGroupBox(trUtf8("Información"));

    lect_inicial = new QLCDNumber();
    lect_inicial->display(0);
    lb_lect_inicial = new QLabel(trUtf8("Lectura inicial:"));
    lb_lect_inicial->setBuddy(lect_inicial);

    lect_final = new QLCDNumber();
    lect_final->display(0);
    lb_lect_final = new QLabel(trUtf8("Lectura final:"));
    lb_lect_final->setBuddy(lect_final);

    kw_h = new QLCDNumber();
    kw_h->display(0);
    lb_kw_h = new QLabel(trUtf8("Kw/h consumidos:"));
    lb_kw_h->setBuddy(kw_h);

    importe_lb = new QLabel("0.00 $");
    lb_importe = new QLabel(trUtf8("Importe:"));
    lb_importe->setBuddy(importe_lb);

    aprox_men_lb = new QLabel("0.00 $");
    lb_aprox_men = new QLabel(trUtf8("Aproximado mensual:"));
    lb_aprox_men->setBuddy(aprox_men_lb);

    QGridLayout *outerLayout = new QGridLayout;
    outerLayout->addWidget(lb_lect_inicial, 0, 0);
    outerLayout->addWidget(lect_inicial, 0, 1);
    outerLayout->addWidget(lb_lect_final, 1, 0);
    outerLayout->addWidget(lect_final, 1, 1);
    outerLayout->addWidget(lb_kw_h, 2, 0);
    outerLayout->addWidget(kw_h, 2, 1);

    outerLayout->addWidget(lb_importe, 4, 0);
    outerLayout->addWidget(importe_lb, 4, 1);
    outerLayout->addWidget(lb_aprox_men, 5, 0);
    outerLayout->addWidget(aprox_men_lb, 5, 1);
    detailsGroupBox->setLayout(outerLayout);

}

void View::Management::RegCEl::createRegistGroupBox()
{
    registGroupBox = new QGroupBox(trUtf8("Registros"));

    createRegistWidgets();

    QGridLayout *registBoxLayout = new QGridLayout;
    registBoxLayout->addWidget(_registsTableView, 0, 0, 1, 6);
    registBoxLayout->addWidget(_addRegistButton, 1, 0, 1, 1);
    registBoxLayout->addWidget(_modRegistButton, 1, 1, 1, 1);
    registBoxLayout->addWidget(_delRegistButton, 1, 2, 1, 1);
    registBoxLayout->addWidget(_printButton, 1, 5, 1, 1);
    registGroupBox->setLayout(registBoxLayout);
}
void View::Management::RegCEl::createRegistWidgets()
{
    _registsTableView = new QTableView;
    _registModel = new RegistModel(Model::Management::RegistManager::getAll());
    _registsTableView ->setModel(_registModel);
    _registsTableView -> setAlternatingRowColors(true);
    _registsTableView -> setShowGrid(false);
    _registsTableView -> setColumnWidth(ColumnRegistId, 75);
    _registsTableView -> setColumnWidth(ColumnRegistDate, 100);
    _registsTableView -> setColumnWidth(ColumnRegistTime, 100);
    _registsTableView -> setColumnWidth(ColumnRegistLectI, 75);
    _registsTableView -> setColumnWidth(ColumnRegistLectF, 75);
    _registsTableView -> setColumnWidth(ColumnRegistKwh, 75);
    _registsTableView -> setColumnWidth(ColumnRegistImport, 75);
    _registsTableView->setColumnHidden(ColumnRegistId, true);
    
     
    _registsTableView -> setSelectionMode(QAbstractItemView::SingleSelection);
    _registsTableView -> setSelectionBehavior(QAbstractItemView::SelectRows);
    _registsTableView -> setEditTriggers(QAbstractItemView::NoEditTriggers);
    _registsTableView -> setFocusPolicy(Qt::NoFocus);

    _addRegistButton = new QPushButton(trUtf8("&Añadir"));
    _addRegistButton -> setIcon(QIcon(":/images/add.png"));

    _modRegistButton = new QPushButton(trUtf8("&Editar"));
    _modRegistButton -> setIcon(QIcon(":/images/modify.png"));
    _modRegistButton -> setEnabled(false);

    _delRegistButton = new QPushButton(trUtf8("&Borrar"));
    _delRegistButton -> setIcon(QIcon(":/images/delete.png"));
    _delRegistButton -> setEnabled(false);

    _printButton = new QPushButton(trUtf8("&Imprimir"));
    _printButton -> setIcon(QIcon(":/images/printing.png"));
    _printButton->setEnabled(false);
}

bool View::Management::RegCEl::verifyDeleteRegist()
{
    return QMessageBox::question(this, trUtf8("Verificar Eliminación del registro"),
                                       trUtf8("¿Está seguro de que quiere eliminar el registro?"),
                                       QMessageBox::Yes | QMessageBox::Default |
                                       QMessageBox::No) == QMessageBox::Yes;
}
void View::Management::RegCEl::createConnections()
{
    connect(_registsTableView -> selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
            this, SLOT(rowSelectionChangedOnRegistsTableView()));

    connect(_registsTableView, SIGNAL(doubleClicked(QModelIndex)),
            this, SLOT(modRegist()));

    connect(_addRegistButton, SIGNAL(clicked()),
            this, SLOT(addRegist()));
    connect(_modRegistButton, SIGNAL(clicked()),
            this, SLOT(modRegist()));
    connect(_delRegistButton, SIGNAL(clicked()),
            this, SLOT(delRegist()));

}
