/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include "registmodel.h"
#include "regist.h"
#include "persistencemanager.h"
#include "types.h"

View::Management::RegistModel::~RegistModel()
{
    if(_regists) {
        foreach(Model::Domain::Regist *regist, *_regists)
            delete regist;

        delete _regists;
    }
}

QList<Model::Domain::Regist *> *View::Management::RegistModel::regists()
{
    return _regists;
}

void View::Management::RegistModel::setRegists(QList<Model::Domain::Regist *> *regists)
{
    _regists = regists;
    reset();
}

bool View::Management::RegistModel::insertRegist(int k, Model::Domain::Regist *regist)
{
    if(k < 0  || k > _regists -> size())
        return false;

    _regists -> insert(k, regist);
    reset();

    return true;
}

bool View::Management::RegistModel::modifyRegist(int k)
{
    if(k < 0  || k > _regists -> size())
        return false;

    reset();

    return true;
}

bool View::Management::RegistModel::removeRegist(int k)
{
    if(k < 0  || k > _regists -> size())
        return false;

    delete _regists -> at(k);
    _regists -> removeAt(k);
    reset();

    return true;
}

int View::Management::RegistModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);

    return _regists -> size();
}

int View::Management::RegistModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);

    return ColumnRegistCount;
}

QVariant View::Management::RegistModel::data(const QModelIndex &index, int role) const
{
    if(index.isValid()) {
        if(role == Qt::TextAlignmentRole) {
            switch(index.column()) {
            case ColumnRegistId:
                return int(Qt::AlignCenter);
            case ColumnRegistDate:
                return int(Qt::AlignLeft | Qt::AlignVCenter);
            case ColumnRegistTime:
                return int(Qt::AlignVCenter);
            case ColumnRegistLectI:
                return int(Qt::AlignVCenter);
            case ColumnRegistLectF:
                return int(Qt::AlignVCenter);
            case ColumnRegistKwh:
               return int(Qt::AlignVCenter);
            default:
                return int(Qt::AlignRight | Qt::AlignVCenter);
                
            }
            
        } else if(role == Qt::DisplayRole) {
            Model::Domain::Regist *regist = _regists -> at(index.row());
            switch(index.column()) {
            case ColumnRegistId:
                return QString::number(regist -> id());
            case ColumnRegistDate:
                return QDate(regist -> date()).toString(DATE_FORMAT);
            case ColumnRegistTime:
                return QTime(regist -> time()).toString(TIME_FORMAT);
            case ColumnRegistLectI:
                return regist -> lecti();
            case ColumnRegistLectF:
                return regist -> lectf();
            case ColumnRegistKwh:
                return regist -> kwh();
            case ColumnRegistImport:
                return QString("%1 $").arg(QString::number(regist -> import(), 'f', MAX_MONEY_PRECISION));
            }
        }
    }

    return QVariant();
}

QVariant View::Management::RegistModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if(role == Qt::DisplayRole) {
        if(orientation == Qt::Vertical)
            return QString::number(section + 1);
        else {
            switch(section) {
            case ColumnRegistId:
                return QObject::trUtf8("ID");
            case ColumnRegistDate:
                return QObject::trUtf8("Fecha");
            case ColumnRegistTime:
                return QObject::trUtf8("Hora");
            case ColumnRegistLectI:
                return QObject::trUtf8("L. inicial");
            case ColumnRegistLectF:
                return QObject::trUtf8("L. final");
            case ColumnRegistKwh:
                return QObject::trUtf8("Kw/h");
            case ColumnRegistImport:
                return QObject::trUtf8("Importe");
            }
        }
    }

    return QVariant();
}
